require 'spec_helper'
describe Mail2cb::EmailHandler do

  before(:each) do
    ENV.delete("AWS_ACCESS_KEY")
    ENV.delete("AWS_SECRET_KEY")
    ENV.delete("AWS_MAILROOM_BUCKET")

    @simple = sample_mail("reply")
    @bare_bones = sample_mail("bare_bones")
    @multipart_related = sample_mail("reply_message_id_in_body")
    @email = Mail.new(@simple)
    @handler = Mail2cb::EmailHandler.new(:inbound_message => @simple, :mailbox => {:account_code => "test_account_code"})
  end

  it "verifies :inbound_message is passed in" do
    expect{Mail2cb::EmailHandler.new(:mailbox => nil)}.to raise_error(RuntimeError, "Require :inbound_message to be passed in")
  end
  it "verifies :mailbox is passed in" do
    expect{Mail2cb::EmailHandler.new(:inbound_message => nil)}.to raise_error(RuntimeError, "Require :mailbox to be passed in")
  end

  it "parses the email to an email object" do
    email = @handler.email

    expect(email).to be_an_instance_of Mail::Message
    expect(email.from).to eq ["stewart@emergeadapt.com"]

  end

  describe "message hash for commsroom" do
    it "generates valid message hash" do
      expected_hash = {
        "from" => {
          :email => "",
          :display_name => "",
          :account_code => ""},
        "subject" => "",
        "sent_at" => "",
        "received_at" => "",
        "body" => "",
        "attachments"=>[
          {
            "filename"=>"",
            "filepath"=>"",
            "storage_type"=>nil
          }
        ],
        "metadata" => {
          "has_bounced" => "",
          "in_reply_to" => "",
          "message_id" => "",
          "account_code" => "",
          "references"=>[],
          "conversation_id"=>""
        },
        "to" => [
          {
            :email=>"",
            :display_name=>"",
            :account_code=>""
          }
        ]
      }
      expect(@handler.generate_message_hash).to match_keys_to(expected_hash)
    end
  end

  describe "recipients" do
    it "are formatted correctly" do
      expected_hash = {:email=>"stewart@emergeadapt.com",:display_name=>"Stewart McKee",:account_code=>"test_account_code"}

      address = Mail::Address.new("Stewart McKee <stewart@emergeadapt.com>")
      expect(@handler.format_recipient(address)).to eq expected_hash
    end
    it "handle comments in local" do
      expected_hash = {:email=>"stewart@emergeadapt.com",:display_name=>"Stewart McKee",:account_code=>"test_account_code"}

      address = Mail::Address.new("Stewart McKee (coder) <stewart@emergeadapt.com>")
      expect(@handler.format_recipient(address)).to eq expected_hash
    end
    it "handle comments in domain" do
      expected_hash = {:email=>"stewart@emergeadapt.com",:display_name=>"Stewart McKee",:account_code=>"test_account_code"}

      address = Mail::Address.new("Stewart McKee <stewart@emergeadapt(super_company).com>")
      expect(@handler.format_recipient(address)).to eq expected_hash
    end
  end

  describe "message metadata" do
    it "extracts the correct metadata from mail" do
      expected_hash = {"has_bounced"=>nil, "in_reply_to"=>"conversation.570f77bc81e9a825e2000001.570f7a1381e9a82659000001@mail.caseblocks.com", "message_id"=>"etPan.570f7a6d.7b5aa127.137@emergeadapt.com", "account_code"=>"test_account_code", "references"=>["conversation.570f77bc81e9a825e2000001.570f7a1381e9a82659000001@mail.caseblocks.com"], "conversation_id"=>"570f77bc81e9a825e2000001"}
      expect(@handler.fetch_metadata).to eq (expected_hash)
    end
    it "formats the metadata correctly" do
      expected_hash = {"has_bounced"=>"", "in_reply_to"=>"", "message_id"=>"", "account_code"=>"", "references"=>[], "conversation_id"=>""}
      expect(@handler.fetch_metadata).to match_keys_to (expected_hash)
    end
    it "includes metadata even if not available" do
      handler = Mail2cb::EmailHandler.new(:inbound_message => @bare_bones, :mailbox => {:account_code => "test_account_code"})
      expected_hash = {"has_bounced"=>nil, "in_reply_to"=>nil, "message_id"=>nil, "account_code"=>"test_account_code", "references"=>[]}
      expect(handler.fetch_metadata).to match_keys_to (expected_hash)
      expect(handler.fetch_metadata).to eq (expected_hash)
    end
  end

  describe "handles attachments" do
    before(:each) do
      @handler = Mail2cb::EmailHandler.new(:inbound_message => @multipart_related, :mailbox => {:account_code => "test_account_code"})
    end

    it "responses with an array of attachments" do
      expect(@handler.generate_message_hash["attachments"]).to be_kind_of(Array)
    end

    context "s3" do
      before(:each) do
        ENV["AWS_ACCESS_KEY"] = "test-aws-access-key"
        ENV["AWS_SECRET_KEY"] = "test-aws-secret-key"
        ENV["AWS_MAILROOM_BUCKET"] = "a-test-bucket-on-s3"

        @attachment = @handler.generate_message_hash["attachments"][0]
      end
      it "formats the hash correctly" do
        expect(@attachment).to match_keys_to({"storage_type" => nil, "filename" => nil, "filepath" => nil, "url" => nil})
      end
      it "sets the type correctly" do
        expect(@attachment["storage_type"]).to eq "S3"
      end
      it "constructs valid path to store attachments in" do
        path = "/a-test-bucket-on-s3/attachments/test_account_code/ab7bec99ece9a5cbf49f40d57d681822"
        expect(@attachment["filepath"].split("/")[0..-2].join("/")).to eq path
      end
      it "calls s3 store" do
        expect(AWS::S3::S3Object).to receive(:store).once.with("/attachments/test_account_code/ab7bec99ece9a5cbf49f40d57d681822/D69E1746-1CEE-47FE-B7E2-8C64E6EC2C63.png", anything(), "a-test-bucket-on-s3")
        @attachment = @handler.generate_message_hash["attachments"][0]
      end
      it "calls s3 url_for" do
        expect(AWS::S3::S3Object).to receive(:url_for).once.with("/attachments/test_account_code/ab7bec99ece9a5cbf49f40d57d681822/D69E1746-1CEE-47FE-B7E2-8C64E6EC2C63.png", 'a-test-bucket-on-s3', {:expires_in => (60 * 60 * 24)})
        @attachment = @handler.generate_message_hash["attachments"][0]
      end
    end

    context "file" do
      before(:each) do
        @attachment = @handler.generate_message_hash["attachments"][0]
      end
      it "formats the hash correctly" do
        expect(@attachment).to match_keys_to({"storage_type" => nil, "filename" => nil, "filepath" => nil})
      end
      it "sets the type correctly" do
        expect(@attachment["storage_type"]).to eq "FILE"
      end
      it "constructs valid folder to store attachments in" do
        path = @attachment["filepath"].split("/")[0..-2].join("/")
        expect(File.directory?(path)).to be_truthy
      end
      it "saves the attachments to disk" do
        expect(File.exists?(@attachment["filepath"])).to be_truthy
      end
    end
  end

  it "works out default format of mail with html" do
    handler = Mail2cb::EmailHandler.new(:inbound_message => @simple, :mailbox => {:account_code => "test_account_code"})
    expect(handler.content_type).to eq "text/html"
  end
  it "works out default format of mail with text only" do
    handler = Mail2cb::EmailHandler.new(:inbound_message => @bare_bones, :mailbox => {:account_code => "test_account_code"})
    expect(handler.content_type).to eq "text/plain"
  end
  it "calls sanitize with expected data" do
    body = "<html><head></head><body><h1>Sample html content</h1></body></html>"
    expect(Sanitize).to receive(:fragment).with(body, Sanitize::Config::RELAXED)
    @handler.sanitize!
  end
  it "calls clean_replies with expected data" do
    body = "<html><head></head><body><h1>Sample html content</h1></body></html>"
    expect(Mail2cb::EmailBodyCleanser).to receive(:parse).with(body, "text/html")
    @handler.clean_replies!
  end
  describe "queuing" do
    it "queues the mail on the apppropriate redis key" do
      $redis = double(:redis_client)
      expect($redis).to receive(:lpush).with("cb:comms:test_account_code:inbox", anything())

      @handler.queue!
    end
  end
end
